mod tests;

use clap::{Parser, Subcommand};

#[derive(Parser)]
#[clap(name = "Bit Goblin Benchmark", author, version, about = "Bit Goblin's hardware benchmarking tool.", long_about = None)]
#[clap(propagate_version = true)]
struct Cli {
    #[clap(subcommand)]
    command: Commands,
}

#[derive(Subcommand)]
enum Commands {
    // disk tests subcommand
    #[clap(name = "disk", about = "Hard drive and SSD benchmarks.")]
    Disk(Disk),
    // network tests subcommand
    #[clap(name = "network", about = "Test various aspects of your network.")]
    Net(Net),
}

#[derive(Parser)]
struct Disk {
    #[structopt(subcommand)]
    disk_commands: DiskCommands,
}

#[derive(Subcommand)]
enum DiskCommands {
    #[clap(name = "write-test", about = "Write a large file to determine sequential disk speeds.")]
    WriteTest {
        #[clap(short = 't', default_value_t = String::from("/tmp/disk-test.tmp"))]
        tempfile: String,
        #[clap(short = 's', default_value_t = 5)]
        size: u8,
    },
}

#[derive(Parser)]
struct Net {
    #[structopt(subcommand)]
    net_commands: NetCommands,
}

#[derive(Subcommand)]
enum NetCommands {
    // ping subcommand
    #[clap(name = "ping", about = "Ping a host to determine network latency.")]
    Ping {
        #[clap(short = 't', default_value_t = String::from("8.8.8.8"))]
        host: String,
        #[clap(short = 'c', default_value_t = 30)]
        count: u16,
    },

    // bandwidth test subcommand
    #[clap(name = "bandwidth", about = "Downloads a remote file to determine network bandwidth.")]
    Bandwidth {
        #[clap(short = 'd', default_value_t = String::from("https://www.bitgoblin.tech/hardware-tests/export-01.mp4"))]
        download: String,
        #[clap(short = 'o', default_value_t = String::from("./tempfile"))]
        output: String,
    },
}

fn main() {
    let cli = Cli::parse();

    // map subcommands back to the main command
    match &cli.command {
        Commands::Disk(args) => match &args.disk_commands {
            DiskCommands::WriteTest { tempfile, size } => tests::disk::disk_write_test(tempfile, size),
        }

        Commands::Net(args) => match &args.net_commands {
            NetCommands::Ping { host, count } => tests::network::ping_host(host, count),
            NetCommands::Bandwidth { download, output } => tests::network::bandwidth_test(download, output),
        },
    }
}
